/* ligand2_chan.c */

/* Avrama Blackwell, GMU, October 1997
   modified from two-state-recept0.c 

   ** modified on 1-13-99 to allow for T-R2 to directly transform into 
   unbound receptor.  Thus, added rate  constants gamma and delta **

** old T-R2' equation:

   T-R2' = alpha T-R1 + -beta T-R2

** old a21, a22, b2:

a21 = alpha
a22 = - beta
b2 = 0

*/

/* This function computes channel conductance of a two state kinetic
   channel which is voltage and ligand dependent.  It computes the 
   fraction of receptors in each state; the channel conductance and current
   is computed from the fraction of receptors in the conducting state.

   This function was designed initially to compute GABA induced chloride currents 
    in the Hermissenda B cell, as measured in Alkon et al., 1992 PNAS.
   Assumptions: Channel protein for activation is ligand-gated
    	        No magnesium block or other co-factors.
                No inactivation gate or desensitization.
*/   
/* Equations implemented in this function:

                                  k1      alpha
   Transmitter/Ligand + Receptor <-> T-R1 <-> T-R2
                                  k2      beta
			^                       ^
                        |                       |
			|          gamma        |
			 _______________________
			           delta
                          
 This biochemical reaction equation leads to the following differential eqn:
 
   T-R1' =  (-alpha - k2 - k1 T) T-R1  + (beta - k1 T) T-R2 + k1 T

   T-R2' = (alpha - delta T) T-R1 + (-beta - gamma - delta T) T-R2 + delta T
      
 Put this in the form of a matrix solution:
 
TR' = A TR + B     with 

a11 = -alpha - k2 - k1 T
a12 = beta - k1 T
a21 = alpha - delta T
a22 = -beta - gamma - delta T
b1 = k1 T
b2 = delta T
*/

#include "chan_ext.h"

#define	Field(F) (channel->F)

ligand2_chan (channel,action)

register struct ligand2_chan_type *channel;
Action		*action;
{
double 	v;			/* Voltage from compartment */
double  ligand;			/* ligand concentration */
double  eff_lig;                /* ligand**rxn_ord */
int 	int_method = -1;	/* Forward Euler */
double 	state1, state2;		/* temp storage of TR1 and TR2 */
double 	TR1pr, TR2pr;		/* derivatives of TR1 and TR2 */
double 	dt;			/* time step */
MsgIn	*msg;
int     n;                      /* for counting messages */

  if (debug > 1) {
     ActionHeader("ligand2_chan", channel,action);
  }

    SELECT_ACTION(action)
    {
    case INIT:
	channel->activation = 0;
	break;
    
    case PROCESS:
    
 /* Get the current voltage and transmitter concentration */	
	v = 0;
	ligand = 0;
	
	MSGLOOP(channel,msg)
	{
	case 0:		
		v = MSGVALUE(msg,0);
		Field(Vm) = v; 
		break;
	
	case 1:
		ligand = MSGVALUE(msg,0);
		Field(ligand) = ligand;
		break;
	}

    	dt = Clockrate(channel);

/* calculate voltage dependent parts of alpha and beta from v	
   For voltage independence, make min - desired value and max = 0*/

	vdep_rate_const (&channel->alpha.rate_const, v, 
		channel->alpha.min,
		channel->alpha.max,
		channel->alpha.v0,
		channel->alpha.slope,
		channel->alpha.in_exp_power,
		channel->alpha.out_exp_power,
		channel->alpha.in_exp_offset,
		channel->alpha.out_exp_offset);
		
	vdep_rate_const (&channel->beta.rate_const, v, 
		channel->beta.min,
		channel->beta.max,
		channel->beta.v0,
		channel->beta.slope,
		channel->beta.in_exp_power,
		channel->beta.out_exp_power,
		channel->beta.in_exp_offset,
		channel->beta.out_exp_offset);

	vdep_rate_const (&channel->k1.rate_const, v, 
		channel->k1.min,
		channel->k1.max,
		channel->k1.v0,
		channel->k1.slope,
		channel->k1.in_exp_power,
		channel->k1.out_exp_power,
		channel->k1.in_exp_offset,
		channel->k1.out_exp_offset);

	vdep_rate_const (&channel->k2.rate_const, v, 
		channel->k2.min,
		channel->k2.max,
		channel->k2.v0,
		channel->k2.slope,
		channel->k2.in_exp_power,
		channel->k2.out_exp_power,
		channel->k2.in_exp_offset,
		channel->k2.out_exp_offset);
		
	vdep_rate_const (&channel->gamma.rate_const, v, 
		channel->gamma.min,
		channel->gamma.max,
		channel->gamma.v0,
		channel->gamma.slope,
		channel->gamma.in_exp_power,
		channel->gamma.out_exp_power,
		channel->gamma.in_exp_offset,
		channel->gamma.out_exp_offset);
		
	vdep_rate_const (&channel->delta.rate_const, v, 
		channel->delta.min,
		channel->delta.max,
		channel->delta.v0,
		channel->delta.slope,
		channel->delta.in_exp_power,
		channel->delta.out_exp_power,
		channel->delta.in_exp_offset,
		channel->delta.out_exp_offset);

/* Calculate the matrix values */

        if (channel->rxn_ord > 1) 
          {
	   eff_lig = pow(ligand,channel->rxn_ord);
          } 
        else 
          {
           eff_lig = ligand;
          }

	channel->a11 = -channel->alpha.rate_const - 
		channel->k2.rate_const - channel->k1.rate_const * eff_lig;
	channel->a12 = channel->beta.rate_const - channel->k1.rate_const * eff_lig;
	channel->b1 = channel->k1.rate_const * eff_lig;

/*	channel->a21 = channel->alpha.rate_const;
	channel->a22 = -channel->beta.rate_const;  */

/* New equations for a21 and a22 and b2 */

	channel->a21 = channel->alpha.rate_const - 
	  channel->delta.rate_const * eff_lig;
	channel->a22 = -channel->beta.rate_const - channel->gamma.rate_const -
	  channel->delta.rate_const * eff_lig;
	channel->b2 = channel->delta.rate_const * eff_lig;

/* Calculate the fraction of state TR1 and TR2 */
/* first, calculate the derivative */

	TR1pr = channel->a11 * channel->TR1 + 
	   channel->a12 * channel->TR2 + channel->b1;
	TR2pr = channel->a21 * channel->TR1 + 
	  channel->a22 * channel->TR2 + channel->b2;

	if (debug > 1)
	printf ("2 state: TR1=%f TR2=%f TR1'=%f TR2'=%f\n", 
		channel->TR1, channel->TR2, TR1pr, TR2pr);
	
/* next, pass the derivative, dt, and the previous state to the
 integration method. */

	state1 = channel->TR1;
	state2 = channel->TR2;
	
/* Evaluate derivative numerically using several methods */	

	if (channel->object->method >= 2 && channel->object->method <= 5) /* Adams-Bashforth */ 
	  {
	    channel->TR1 = IntegrateMethod (channel->object->method, channel, state1, TR1pr, 0, dt, "TR1");
	    channel->TR2 = IntegrateMethod (channel->object->method, channel, state2, TR2pr, 0, dt, "TR2");
	  }
	else /* use forward Euler because can't use implicit methods */  
	  {
	    channel->TR1 = DirectIntegrate (int_method, channel, state1, TR1pr, dt, "TR1");
	    channel->TR2 =  DirectIntegrate(int_method, channel, state2, TR2pr, dt, "TR2");
	  }
	if (debug > 1)
	printf ("After Integration: TR1=%f TR2=%f \n", 
		channel->TR1, channel->TR2);
	
/* Calculate the conductance and current */

	if (channel->cond_state == 1)
	   {
		channel->G = channel->TR1* channel->Gbar;
		channel->I = (v - channel->Vr) * channel->G;
	   }
	else
	   {
		channel->G = channel->TR2* channel->Gbar;
		channel->I = (v - channel->Vr) * channel->G;
	   }

	break;
    
    case RESET:

 /* Get the current voltage and transmitter concentration */	
	v = 0;
	ligand = 0;
	
	MSGLOOP(channel,msg)
	{
	case 0:		
		v = MSGVALUE(msg,0);
		Field(Vm) = v; 
		break;
	
	case 1:
		ligand = MSGVALUE(msg,0);
		Field(ligand) = ligand;
		break;
	}

/* calculate the forward and backward rate constants from ligand and v 	*/

	vdep_rate_const (&channel->alpha.rate_const, v, 
		channel->alpha.min,
		channel->alpha.max,
		channel->alpha.v0,
		channel->alpha.slope,
		channel->alpha.in_exp_power,
		channel->alpha.out_exp_power,
		channel->alpha.in_exp_offset,
		channel->alpha.out_exp_offset);
		

	vdep_rate_const (&channel->beta.rate_const, v, 
		channel->beta.min,
		channel->beta.max,
		channel->beta.v0,
		channel->beta.slope,
		channel->beta.in_exp_power,
		channel->beta.out_exp_power,
		channel->beta.in_exp_offset,
		channel->beta.out_exp_offset);

	vdep_rate_const (&channel->k1.rate_const, v, 
		channel->k1.min,
		channel->k1.max,
		channel->k1.v0,
		channel->k1.slope,
		channel->k1.in_exp_power,
		channel->k1.out_exp_power,
		channel->k1.in_exp_offset,
		channel->k1.out_exp_offset);

	vdep_rate_const (&channel->k2.rate_const, v, 
		channel->k2.min,
		channel->k2.max,
		channel->k2.v0,
		channel->k2.slope,
		channel->k2.in_exp_power,
		channel->k2.out_exp_power,
		channel->k2.in_exp_offset,
		channel->k2.out_exp_offset);


	vdep_rate_const (&channel->gamma.rate_const, v, 
		channel->gamma.min,
		channel->gamma.max,
		channel->gamma.v0,
		channel->gamma.slope,
		channel->gamma.in_exp_power,
		channel->gamma.out_exp_power,
		channel->gamma.in_exp_offset,
		channel->gamma.out_exp_offset);
		
	vdep_rate_const (&channel->delta.rate_const, v, 
		channel->delta.min,
		channel->delta.max,
		channel->delta.v0,
		channel->delta.slope,
		channel->delta.in_exp_power,
		channel->delta.out_exp_power,
		channel->delta.in_exp_offset,
		channel->delta.out_exp_offset);

/* Calculate the matrix values */

        if (channel->rxn_ord > 1) {
	   eff_lig = pow(ligand,channel->rxn_ord);
        } else {
           eff_lig = ligand;
	}

	channel->a11 = -channel->alpha.rate_const - 
		channel->k2.rate_const - channel->k1.rate_const * eff_lig;
	channel->a12 = channel->beta.rate_const - channel->k1.rate_const * eff_lig;
	channel->b1 = channel->k1.rate_const * eff_lig;

/*	channel->a21 = channel->alpha.rate_const;
	channel->a22 = -channel->beta.rate_const;  */

/* New equations for a21 and a22 and b2 */

	channel->a21 = channel->alpha.rate_const - 
	  channel->delta.rate_const * eff_lig;
	channel->a22 = -channel->beta.rate_const - channel->gamma.rate_const -
	  channel->delta.rate_const * eff_lig;
	channel->b2 = channel->delta.rate_const * eff_lig;

/* Calculate the steady state values of the channel */

/*		channel->TR1 = -Field(b1) * Field(a22) / 
			(Field(a21) * Field(a12) - Field(a22) * Field(a11));

		channel->TR2 = Field(b1) * Field(a21) / 
			(Field(a21) * Field(a12) - Field(a22) * Field(a11)); */

	/* New equations as of 1-13-99 */

	    channel->TR1 = Field(b2) * Field(a12) + Field(b1) * Field(a22) / 
			(Field(a22) * Field(a11) - Field(a21) * Field(a12)  );

	    channel->TR2 = Field(b1) * Field(a21) + Field(b2) * Field(a11) / 
			(Field(a21) * Field(a12) - Field(a22) * Field(a11));

/* Calculate the current */

	if (channel->cond_state == 1)
	   {
		channel->G = channel->TR1* channel->Gbar;
		channel->I = (v - channel->Vr) * channel->G;
	    }
	else
	   {
		channel->G = channel->TR2* channel->Gbar;
		channel->I = (v - channel->Vr) * channel->G;
	   }

	break;

    case CHECK:
      n=0;
      MSGLOOP(channel,msg){
        case 0:
	  n +=1;
	  break;
        case 1:
	  n+=1;
	  break;
      }
      if(n != 2) {
	 ErrorMessage(" ligand2_chan","Missing v or ligand msg.",
			      channel);
      }
      break;
  }
}
