#include "chem_ext.h"

/* Implementation of general 1D or 2D concentration pool. 
**  in-flows/outflows include
**		Vdep calcium currents (measured in amps)
**		IP3R calcium currents (dependent on concentration gradient)
**		pumps to cytosol and ER
**		1-D or 2D diffusion
**		general kinetic reactions
** flows between pools are assumed to be computed in terms of # moles and 
** divided by volume within the pool to get the effect on concentration.  
** flows within a single pool can be performed directly on concentration.
**
** This scheme works fine for single step integration methods,
**	multi-step methods will not be as accurate because changes in
**	variables outside this object (e.g. Bfree in poolbuffers) are
**	invisible to the integrator till the next message */

/*  Avrama Blackwell, GMU, Dec 1997 
    Modified Dec 1999 to  compute quantity */

static  double	savedata[2];

int rxnpool(molecule,action)
register struct rxnpool_type *molecule;
Action		*action;
{
double	A,B;		/* integrate dconc/dt = A - B*conc */
double  Aconc, Amoles, Bconc, Bmoles; /*accumulates inputs in units of Moles or concentration */
double	concnew;
double	DV;
double	dt;
MsgIn 	*msg;
int		nc = 0;
int		n;

    if(debug > 1){
	ActionHeader("rxnpool",molecule,action);
    }

    SELECT_ACTION(action){
    case INIT:             /* can perform integration in terms of concentration or quantity */
      if (molecule->type == 0)
	molecule->previous_state = molecule->Conc;
      else 
	molecule->previous_state = molecule->quantity;
	break;

    case PROCESS:
	dt = Clockrate(molecule);
	Aconc = Bconc = Amoles = Bmoles = 0;

	/* Read the msgs to get the rxnpool concentration.
	   Cases 0 - 2 are in units of conc; no need to divide by vol */
	MSGLOOP(molecule,msg){
	    case 0:	/* 0th order rxn */
	      		/* 0 = A, units must be concentration per millisecond */
	    	Aconc += MSGVALUE(msg,0);
	    	break;
	    case 1:	/*1st order rxn, e.g. degradation */
	       		/* 0 = B, units must be "per millisecond" */
	   	Bconc += MSGVALUE(msg,0);
	   	break;
	    case 2:	/* interaction with 2nd order reaction (e.g. buffer) */
	      /* Both A and B have units of concentration / second;  
	       Thus, concentration needs to be removed from B field */	
	                 /* 0 = A */
			/* 1 = B */
		Aconc += MSGVALUE(msg,0);
		if (molecule->Conc != 0)
		  Bconc += MSGVALUE(msg,1)/molecule->Conc;
		break;
	    case 3:	/* interaction with pump */
	      /* pump works between compartments, units are cm/msec.  Multiplying by
	       surface area produces units of cm^3/msec.  Multiplying by concentration 
	      gives units of moles */
			/* 0 = kPump */
			/* 1 = Ceq */
			/* dQ/dt = -kPump*(Co-Ceq) */
		DV = MSGVALUE(msg,0);
		Amoles += DV*MSGVALUE(msg,1);
		Bmoles += DV;
		break;
	    case 4:	/* current in/outflow */
	      /* current / e * Faraday yields moles; convert to 
		 concentration by dividing by volume */
			/* 0 = charge */
			/* 1 = I */
			/* dQ/dt = I/(e*Faraday) */
	      /* Faraday = 96495 Coulombs/mole (C/mol), 
		 convert I to C/time units.  If SI units, Iunits=1
		 If using nA (nC/sec) and msec, 
		 I * 1e-12 C-sec / nC-msec
		 convert from C/mol to C/mmol with F*units
		 Also, put minus sign in because neg currents increase Conc*/
		Amoles -= MSGVALUE(msg,1)*molecule->Iunits /(MSGVALUE(msg,0)*96495*molecule->units);
		break;
	    case 5:	/* diffusion or CICR or reaction in mmole units */
	      /* CICR and diffusion are between compartments, A and B in units 
                 of moles needs to be removed */
			/* 0 = A (pool indep value) */
			/* 1 = B*conc (pool dep value) */
		Amoles += MSGVALUE(msg,0);
		if (molecule->Conc != 0)
		  Bmoles += MSGVALUE(msg,1)/ (molecule->Conc);
		break;
	    case 6:	/* interaction with mmpump or M-M rxn;
			   A units: moles/msec; must divide by volume */
	    		/* 0 = A */
	    	Amoles += MSGVALUE(msg,0);
	    	break;
	    case 7:	/* interaction with 2nd order reaction (e.g. buffer) */ 
	                /* Differs from case 2 in that reaction in mmole units */
	      /* value of pool1 quantity must be removed from B field*/
			/* 0 = A (pool indep value) */
			/* 1 = B*quantaity (pool dep value) */
		Amoles += MSGVALUE(msg,0);
		if (molecule->quantity != 0)
		  Bmoles += MSGVALUE(msg,1)/ (molecule->quantity);
		break;
	}

/* After accumulating inflows and outflows, integrate to get new  conc. */
	if (molecule->type == 0)
	  {
	    A = Amoles/molecule->vol + Aconc;
	    B = Bmoles/molecule->vol + Bconc;
	    molecule->Conc = IntegrateMethod((B < 1.0e-10) ?
			FEULER_INT: molecule->object->method, 
				molecule,molecule->Conc,A,B,dt,"concentration");

	    /* Do not allow Co to get lower tham Cmin, note that this
	    ** limit is only felt indirectly by poolbuffers! */
	    if (molecule->Conc < molecule->Cmin) 
	      molecule->Conc = molecule->Cmin;

	    molecule->quantity = molecule->Conc*molecule->vol;
	  }

	if (molecule->type == 1)
	  {
	    A = Amoles + Aconc*molecule->vol;
	    B = Bmoles + Bconc*molecule->vol;
	    molecule->quantity = IntegrateMethod((B < 1.0e-10) ?
			FEULER_INT: molecule->object->method, 
				molecule,molecule->quantity,A,B,dt,"quantity");

		molecule->Conc = molecule->quantity/molecule->vol;

	    /* Do not allow Co to get lower tham Cmin, note that this
	    ** limit is only felt indirectly by poolbuffers! */
	    if (molecule->Conc < molecule->Cmin) 
	      {
		molecule->Conc = molecule->Cmin;
		molecule->quantity = molecule->Conc*molecule->vol;
	      }
	  }
	break;

    case RESET:
      if (molecule->type == 0)
	{
	  molecule->Conc = molecule->Cinit;
	  molecule->quantity = molecule->Conc*molecule->vol;
	}
      else 
	{
	  molecule->quantity = molecule->Qinit;
	  molecule->Conc = molecule->quantity/molecule->vol;
	}
	break;
    
    case CHECK:
	/* minimum concentration */
	if(molecule->Cmin < 0.0){
	    ErrorMessage("rxnpool", "Invalid Cmin.", molecule);
	}
	if(molecule->Cinit < 0.0){
	    ErrorMessage("rxnpool", "Invalid Cinit.", molecule);
	}
	if(molecule->Qinit < 0.0){
	    ErrorMessage("rxnpool", "Invalid Qinit.", molecule);
	}
	/* length */
	if(molecule->len <= 0.0 && molecule->radius <= 0.0){
	    ErrorMessage("rxnpool", "Invalid Len.", molecule);
	}
	/* volume */
	if(molecule->vol <= 0.0){
	    ErrorMessage("rxnpool", "Invalid Vol.", molecule);
	}
	if((molecule->units == 0.0) || (molecule->Iunits == 0.0)){
	    ErrorMessage("rxnpool", "units=0 or Iunits=0.", molecule);
	}
	break;


    case SAVE2:
	savedata[0] = molecule->Conc;
	savedata[1] = molecule->previous_state;
	/* action->data contains the file pointer */
	n=2;
	fwrite(&n,sizeof(int),1,(FILE*)action->data);
	fwrite(savedata,sizeof(double),2,(FILE*)action->data);
	break;

    case RESTORE2:
	/* action->data contains the file pointer */
	fread(&n,sizeof(int),1,(FILE*)action->data);
	if (n != 2) {
	    ErrorMessage("rxnpool", "Invalid savedata length", molecule);
	    return n;
	}
	fread(savedata,sizeof(double),2,(FILE*)action->data);
	molecule->Conc = savedata[0];
	molecule->previous_state = savedata[1];
	break;

    }

    return(0);
    
}

